# Copyright(c) 2011-2012 Association of Universities for Research in Astronomy, Inc.

procedure gtile (inimages)

# Tile the amplifiers of the GMOS input images into their respective CCDs and
# return the input image as phu plus MDF if presenet, plus 3 (each CCD)
# versions each type of extension (e.g., SCI, VAR, DQ) with appropriate header
# keywords updated. All supplied CCDs are returned by default (out_ccds="all").
# An individual CCD science frame (with no phu) is returned when fl_stats_only
# is yes and out_ccds != "all". If fl_stats_only=yes and fl_tile_det=yes
# the entire detector tiled together can be returned. There are checks for
# flag clashes, where there are clashes flags get reset. - MS

# This task is hidden and designed for use when doing things
# like imstat calls on CCD2 in giflat and gemlocal tasks
# It makes the following assumptions:
#     The detector orientation is the same for North and South; i.e.,
#         CCD1 detsec X1 starts at 1 and CCD3 detsec X2 ends at 6144

char    inimages    {prompt="GMOS images to tile (single/list)"}
char    outimages   {"", prompt="Output image name or list of names"}
char    outpref     {"ti", prompt="prefix for output images"}
char    out_ccds    {"all", prompt="Which CCDs to return. (all|1|2|3)"}
bool    fl_stats_only {no, prompt="Return only the science data for use with statistics tasks"}
bool    fl_tile_det {no, prompt="To tile all three CCDs - only to be used with fl_stats_only=yes"}
bool    fl_pad      {no, prompt="Include chip gaps in tiled detector image?"}
real    padvalue    {0., prompt="Value of chipgap pixels when fl_pad=yes"}
char    chipgap     {"default", prompt="Chipgap width in pixels"}
char    sci_ext     {"SCI", prompt="Name of science extension(s)"}
char    var_ext     {"VAR", prompt="Name of variance extension(s)"}
char    dq_ext      {"DQ", prompt="Name of data quality extension(s)"}
char    mdf_ext     {"MDF", prompt="Mask definition file extension name"}
char    key_detsec  {"DETSEC", prompt="Header keyword for detector section"}
char    key_ccdsec  {"CCDSEC", prompt="Header keyword for CCD section"}
char    key_datasec {"DATASEC", prompt="Header keyword for data section"}
char    key_biassec {"BIASSEC", prompt="Header keyword for overscan section"}
char    key_ccdsum  {"CCDSUM", prompt="Header keyword for CCD binning"}
char    rawpath     {"", prompt="Path to raw data file(s)"}
char    logfile     {"", prompt="Name of logfile to use"}
bool    fl_verbose  {no, prompt="Verbose"}
int     status      {0, prompt="Exit status (0=good)"}
struct  *scanfile   {prompt="For internal use only"}
struct  *scanfileII {prompt="For internal use only"}

begin

    # Setup local variables for input parameters
    char    l_inimages = ""
    char    l_outimages = ""
    char    l_outpref = ""
    char    l_out_ccds = ""
    bool    l_fl_stats_only
    bool    l_fl_tile_det
    bool    l_fl_pad
    real    l_padvalue = 0.
    char    l_chipgap = ""
    char    l_sci_ext = ""
    char    l_var_ext = ""
    char    l_dq_ext = ""
    char    l_mdf_ext = ""
    char    l_key_detsec = ""
    char    l_key_ccdsec = ""
    char    l_key_datasec = ""
    char    l_key_biassec = ""
    char    l_key_ccdsum = ""
    char    l_rawpath = ""
    char    l_logfile = ""
    bool    l_fl_verbose

    # Set up variables used in code
    char    filename, curent_image, inimage_files[200], outimage_files[200]
    char    tmpinimg, tmpoutimg, tmpimg, tmptile, tmpmdf
    char    datasec[12], detsec[12], ccdsec[12], biassec[12], ccdsum
    char    l_detsec, l_ccdsec, l_datasec, l_biassec, extn
    char    extn_list, in_extn, imtile_list, date_info
    char    out_pixtype, inlist, outlist, next_inp, l_sciext
    char    out_ampsname, amplifiername[12], file_of_chipgap
    char    l_sec, l_key, in_tile_file, listfile, instrument, dettype
    char    rphend, pathtest
    int     tx1, tx2, ty1, ty2, ampsorder[12]
    int     detx1[12], detx2[12], dety1[12], dety2[12]
    int     ccdx1[12], ccdx2[12], ccdy1[12], ccdy2[12]
    int     datax1[12], datax2[12], datay1[12], datay2[12]
    int     biasx1[12], biasx2[12], biasy1[12], biasy2[12]
    int     data_width[12], bias_width[12]
    int     maxfiles, junk, in_pixtype, num_images, total_num_amplifiers
    int     number_of_ampsperccd[3]
    int     i, j, k, jj, kk, refextn, naxis1, tmpint, num_ccds, current_extn
    int     jjstart, jjstop, jstart
    int     det_start[3], det_stop[3], ordered_refextn
    int     num_sci_extns, num_extns, num_of_this_extension
    int     detroiy_start, row_removal_value, row_removal_value_binned
    int     xbin, ybin, image_width, image_height, site, iccd
    int     secx1, secx2, secy1, secy2, atlocation, raw_width[2,3], gapwidth
    real    crpix1_in[12], dummy_crpix1
    real    crpix2_in[12], dummy_crpix2, crpix1_out, crpix2_out
    bool    fl_useprefix, mdf_exists
    bool    dotile, ishamamatsu
    bool    debug, istrimmed
    char    array_keys[3] = "DATASEC","CCDSEC","DETSEC"
    char    l_pixtype[12] = "bool","char","short","int","long","real","double","complex","pointer","struct","ushort","ubyte"
    # l_pixtype integers -- > 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 (from iraf$unix/lib/iraf.h)

    # Set up parameters caches of the following tasks
    cache ("gimverify", "gemextn", "gemdate", "imjoin")

    # Set internal debug flag
    debug = no

    # Set default values
    maxfiles = 200
    total_num_amplifiers = 0
    status = 0
    l_sciext = ""
    ishamamatsu = no
    istrimmed = no

    # Set det limits for start and stop of each CCD
    det_start[1] = 1
    det_start[2] = 2049
    det_start[3] = 4097

    det_stop[1] = 2048
    det_stop[2] = 4096
    det_stop[3] = 6144

    # Define the chip gaps for each site and detector in unbinned pixels
    # raw_width[site, iccd]
    # site: 1 = GMOS-N; 2 = GMOS-S
    # iccd: 1 = EEV; 2 = e2vDD; 3 = Hamamatsu
    raw_width [1,1] = 37
    raw_width [1,2] = 37
    raw_width [1,3] = 37
    raw_width [2,1] = 37
    raw_width [2,3] = 37

    # Read input parameters
    junk = fscan (inimages, l_inimages)
    junk = fscan (outimages, l_outimages)
    junk = fscan (outpref, l_outpref)
    l_fl_stats_only = fl_stats_only
    l_fl_tile_det = fl_tile_det
    l_fl_pad = fl_pad
    l_padvalue = padvalue
    l_chipgap = chipgap
    junk = fscan (out_ccds, l_out_ccds)
    junk = fscan (sci_ext, l_sci_ext)
    junk = fscan (var_ext, l_var_ext)
    junk = fscan (dq_ext, l_dq_ext)
    junk = fscan (mdf_ext, l_mdf_ext)
    junk = fscan (key_detsec, l_key_detsec)
    junk = fscan (key_ccdsec, l_key_ccdsec)
    junk = fscan (key_datasec, l_key_datasec)
    junk = fscan (key_biassec, l_key_biassec)
    junk = fscan (key_ccdsum, l_key_ccdsum)
    l_rawpath = rawpath
    junk = fscan (logfile, l_logfile)
    l_fl_verbose = fl_verbose

    ###########
    # Test the logfile here to start logging:
    if ((l_logfile == "") || (stridx(" ", l_logfile) > 0)) {
        l_logfile = gmos.logfile
        if ((l_logfile == "") || (stridx (" ", l_logfile) > 0)) {
            l_logfile = "gmos.log"
            printlog ("GTILE - WARNING: Both gprepare.logfile and "//\
                "gmos.logfile fields are empty.", l_logfile, verbose+)
            printlog ("                   Using default file gmos.log", \
                l_logfile, verbose+)
        }
    }

    # Print start time
    printlog ("", l_logfile, l_fl_verbose)
    gemdate ()
    printlog ("GTILE - Started: "//gemdate.outdate, l_logfile, l_fl_verbose)

    ################
    # Start checks #
    ################

    # Perform checks on input/output lists; setup filenames;

    # Create temporary inlist and outlist files
    inlist = mktemp("inlist")
    outlist = mktemp("outlist")

    #### Rawpath ####
    if ((l_rawpath != "") && (stridx(" ", l_rawpath) == 0)) {
        # Check it has a trailing slash after expaning any environmental
        # variables
        rphend = substr(l_rawpath,strlen(l_rawpath),strlen(l_rawpath))
        if (rphend == "$") {
            show (substr(l_rawpath,1,strlen(l_rawpath)-1)) | scan (pathtest)
            rphend = substr(pathtest,strlen(pathtest),strlen(pathtest))
        }
        if (rphend != "/") {
            l_rawpath = l_rawpath//"/"
        }
        if (!access(l_rawpath)) {
            printlog ("ERROR - GIREDUCE: Cannot access rawpath: "//l_rawpath, \
                l_logfile, verbose+)
            goto clean
        }
    }

    atlocation = 0
    atlocation = stridx("@", l_inimages)
    if (atlocation > 0) {
        listfile = substr(l_inimages,(atlocation + 1),strlen(l_inimages))
        # Check to se if file exists
        if (!access(listfile)) {
            printlog ("ERROR - GTILE: Cannot access file: \""//listfile//\
                "\". Exiting",\
                l_logfile, verbose+)
            status = 2
            goto clean
        }
    }

    # Check for an input list
    if (atlocation >= 1) {
        # Input is a list
        sections (l_inimages, > inlist)
    } else {
        # Images not in an @list
        files (l_inimages, sort=no, > inlist)
    } # End of input list checks

    # Set up parameters for reading input files
    scanfile = inlist
    filename = ""
    num_images = 1

    # Loop over input list of images
    while (fscan(scanfile,filename) != EOF) {
        # Verify image
        gimverify (l_rawpath//filename)

        if (gimverify.status > 0) {
             # Image bad/doesn't exist goto to clean
             printlog ("", l_logfile, verbose+)
             printlog ("GTILE - ERROR: File: "//filename//\
                 " does not exist or is bad", l_logfile, verbose+)
             # Change status to bad
             status = 1
             goto clean
        }
        inimage_files[num_images] = l_rawpath//filename
        num_images += 1

    } # End of while loop verifying images and populating inimage_files[]

    # Fix the num_images counter to be correct
    num_images -= 1

    # Set default flag for use of the prefix
    fl_useprefix = yes

    # Check for an output list
    if (l_outimages != "" && l_outimages != " ") {
        if (stridx("@",l_inimages) >= 1) {
            # Output is a list
            sections (l_outimages, > outlist)
        } else {
            # Images not in an @list
            files (l_outimages, sort=no, > outlist)
        }

        # Check the number of output files is equal to the number of inputs
        sections ("@"//outlist, option="nolist")
        if (num_images == int(sections.nimages)) {
            fl_useprefix = no
        } else {
            delete (outlist, verify-, >& "dev$null")
        }
        # End of if statement checking number of in and out files

    } # End of output list checks

    # Check if prefix is to be used
    if (fl_useprefix) {
        # Use prefix
        sections (l_outpref//"@"//inlist, option="root", > outlist)
    }

    # Set up parameters for reading output file list
    scanfile = outlist
    filename = ""
    i = 1

    # Loop over outlist of images to populate outimage_files and check they
    # don't exist
    while (fscan(scanfile,filename) != EOF) {
        # Verify image
        gimverify (filename)
        if (gimverify.status != 1) {
             # If image doesn't exit
             printlog ("", l_logfile, verbose+)
             printlog ("GTILE - ERROR: File: "//filename//\
                 " already exists", l_logfile, verbose+)
             # Change status to bad
             status = 1
             goto clean
        }
        outimage_files[i] = filename
        i += 1
    } # End of while loop verifying and populating outimage_files[]

    ###########
    # Check some of the flags for clashes
    if (l_fl_stats_only && l_out_ccds == "all" && !l_fl_tile_det) {
        printlog ("", l_logfile, verbose+)
        printlog ("GTILE - WARNING: stats_only flag set to yes, out_ccds "//\
            "is set to all", l_logfile, verbose+)
        printlog ("                 and the tile_det flag is set to no.", \
            l_logfile, verbose+)
        printlog ("                 **Resetting fl_tile_det to yes. Only "//\
            "the science frames", l_logfile, verbose+)
        printlog ("                  will be returned tiled into the "//\
            "the available size of the entire detector.", l_logfile, verbose+)
        printlog ("", l_logfile, verbose+)
        l_fl_tile_det = yes
    }
    if (!l_fl_stats_only && l_fl_tile_det) {
        printlog ("", l_logfile, verbose+)
        printlog ("GTILE - WARNING: fl_stats_only flag set to no and "//\
            "fl_tile_det set to yes.", l_logfile, verbose+)
        printlog ("                 **Resetting fl_stats_only to yes. "//\
            "Only the science frames", l_logfile, verbose+)
        printlog ("                 will be returned tiled into the "//\
            "available size of the entire detector.", l_logfile, verbose+)
        l_fl_stats_only = yes
    }
    if (!l_fl_stats_only && l_out_ccds != "all" && !l_fl_tile_det) {
        printlog ("", l_logfile, verbose+)
        printlog ("GTILE - WARNING: out_ccds not set to all, but "//\
            "fl_stats_only flag is no", l_logfile, verbose+)
        printlog ("                 and the tile_det flag is set to no.", \
            l_logfile, verbose+)
        printlog ("                 **Resetting out_ccds to all. All "//\
            "the extension types", l_logfile, verbose+)
        printlog ("                 will be returned tiled into the "//\
            "the available size of the entire detector.", l_logfile, verbose+)
        printlog ("", l_logfile, verbose+)
        l_out_ccds = "all"
    }
    if (l_out_ccds != "all" && l_fl_tile_det) {
        printlog ("", l_logfile, verbose+)
        printlog ("GTILE - WARNING: out_ccds not set to all, but fl_tile"//\
            "_det flag is yes", l_logfile, verbose+)
        printlog ("                 **Resetting fl_tile_det to no.", \
            l_logfile, verbose+)
        printlog ("", l_logfile, verbose+)
        l_fl_tile_det = no
    }

    #### Chipgap related ####
    # Check chipgap input
    if ((l_chipgap == "") || (stridx(" ",l_chipgap) >= 1)) {
        printlog ("ERROR - GTILE: chipgap parameter not set. Exiting", \
            l_logfile, verbose+)
        status = 1
        goto clean
    } else {
        if (l_chipgap != "default") {
            # Check the input is a number
            gemisnumber (l_chipgap, ttest="decimal", verbose=no)
            if (gemisnumber.status == 0) {
                if (gemisnumber.fl_istype) {
                    gapwidth = int(l_chipgap)
                } else {
                    printlog ("ERROR - GTILE: chipgap parameter is not a \
                        value. Exiting", l_logfile, verbose+)
                    status = 1
                    goto clean
                }
            } else {
                printlog ("ERROR - GTILE: gemisnumber returned a non-zero \
                    status. Exiting", l_logfile, verbose+)
                status = 1
                goto clean
            }
        }
    }

    # Check for conflicting flags
    if (l_fl_pad && (!l_fl_stats_only || (l_out_ccds != "all") \
        && !l_fl_tile_det)) {
        printlog ("", l_logfile, verbose+)
        printlog ("GTILE - WARNING: fl_pad is set to yes but one or more of"//\
             " the following parameters ", \
            l_logfile, verbose+)
        printlog ("                 are not set correctly:", \
            l_logfile, verbose+)
        printlog ("                     fl_stats_only, out_ccds or \
            fl_tile_det", l_logfile, verbose+)
        printlog ("                 They are required to be set as follows"//\
            " to use the fl_pad=yes",
            l_logfile, verbose+)
        printlog ("                 option: ",
            l_logfile, verbose+)
        printlog ("                     fl_stats_only=yes, out_ccds=\"all\" \
            and fl_tile_det=yes", l_logfile, verbose+)
        printlog ("                 **Resetting fl_pad to no.", \
            l_logfile, verbose+)
        l_fl_pad = no
    }
    #################
    # End of checks #
    #################

    ##########################
    # Start of the hard work #
    ##########################

    # Input files are in inimage_files[]
    # Output files are in outimages_files[]
    # Total number of input images is num_files [loop < num_files]

    # Printlog warning on usage
    printlog ("", l_logfile, l_fl_verbose)
    printlog ("GTILE - WARNING: GTILE only tiles the data sections of each"//\
        " type of "//\
        "extension", l_logfile, l_fl_verbose)
    printlog ("                 No overscan sections will be included in "//\
        "the output image", l_logfile, l_fl_verbose)
    printlog ("", l_logfile, l_fl_verbose)

    # Loop over input images
    for (i=1; i<=num_images; i+=1) {

        # Initiate the list of types of extensions to loop over
        extn_list = mktemp("tmpextn_list")

        # Read the instrument name
        keypar (inimage_files[i]//"[0]", "INSTRUME", silent+)
        instrument = keypar.value
        if (instrument == "GMOS-N") {
            # GN
            site = 1
        } else if (instrument == "GMOS-S") {
            # GS
            site = 2
        } else {
            printlog ("ERROR - GTILE: Unrecognizable INSTRUME keyword value"//\
                instrument//" for image "//inimage_files[i]//". Exiting", \
            l_logfile, verbose+)
            status = 1
            goto clean
        }

        # Read the detector type
        keypar (inimage_files[i]//"[0]", "DETTYPE", silent+)
        dettype = keypar.value
        if (dettype == "S10892-01") {
            # Hamamatsu CCDs
            ishamamatsu = yes
            row_removal_value = 48
            iccd = 3
        } else if (dettype == "SDSU II e2v DD CCD42-90") {
            # e2vDD CCDs
            iccd = 2
        } else if (dettype == "SDSU II CCD") {
            # EEV CCDs
            iccd = 1
        } else {
            printlog ("ERROR - GTILE: Unrecognizable DETTYPE keyword value"//\
                dettype//" for image "//inimage_files[i]//". Exiting", \
            l_logfile, verbose+)
            status = 1
            goto clean
        }

        # Set mdf_exists flag
        mdf_exists = no

        # Check if greduce'd
        keypar (inimage_files[i]//"[0]", "GPREPARE", silent=yes)
        if (!keypar.found) {
            # Set the science extension name
            l_sciext = ""

            # Read the number of input science extensions
            fxhead (inimage_files[i], format_file="", long_head-,
                count_lines-, >& "dev$null")
            fxhead (inimage_files[i], format_file="", long_head-, \
                count_lines-) | count ("STDIN") | scan (next_inp)
            fxhead (inimage_files[i], format_file="", long_head-, \
                count_lines-) | fields ("STDIN", "1", lines=str(next_inp),\
                quit_if_miss-, print_file-) | scan (next_inp)

            # Set the total number of amplifiers used
            total_num_amplifiers = int(next_inp)

            if (debug) {
                printlog ("Total number of amplifiers "//total_num_amplifiers,\
                    l_logfile, verbose+)
            }

            # Set up list of extensions to loop over
            print (l_sciext, > extn_list)

        } else {

            # Set the science extension name
            l_sciext = l_sci_ext//","

            # Determine the number of amplifiers
            gemextn (inimage_files[i],process="expand", \
                check="exists", extver="1-", \
                extname=l_sci_ext, \
                outfile="dev$null")
            if (gemextn.count > 0) {
                total_num_amplifiers = gemextn.count
            }

            # Set up list of extensions to loop over
            print (l_sciext, > extn_list)

            # Check if stats_only, if not -  see if vardq planes and MDF exist
            # If so, set up the list to loop over properly
            if (!l_fl_stats_only) {

                # Check for VARDQ planes exist
                gemextn (inimage_files[i],process="expand", \
                    check="exists", extver="1-", \
                    extname=l_var_ext//","//l_dq_ext, \
                    outfile="dev$null")
                if (gemextn.count > 0) {
                    printlog ("GTILE - Variance and DQ planes found", \
                        l_logfile, l_fl_verbose)
                    printlog ("", l_logfile, l_fl_verbose)
                    print ("VAR,", >> extn_list)
                    print ("DQ,", >> extn_list)
                }

                # Find any MDFs
                tinfo (inimage_files[i]//"["//l_mdf_ext//"]", ttout-, >& \
                    "dev$null")
                if (tinfo.tbltype == "fits") {
                    printlog ("GTILE - MDF found", l_logfile, l_fl_verbose)
                    mdf_exists = yes
                }
            } # End of if l_fl_stats_only flag check
        }

        # Read the binning for this image - gets re-read a couple of times
        # later on too - MS
        keypar (inimage_files[i]//"["//l_sciext//"1]",
            l_key_ccdsum, silent+)
        if (keypar.found) {
            print (keypar.value) | scanf ("%d %d", xbin, ybin)
            if (debug) {
                print ("XBIN: "//xbin//"YBIN: "//ybin, l_logfile, \
                    verbose+)
            }
        } else {
            xbin = 1
            ybin = 1
        }

        # To get to here with l_fl_pad true all of the correct flags must have
        # been set appropriately
        if (l_fl_pad) {
            # Set the gap width if default - already set if user defined
            if (l_chipgap == "default") {
                gapwidth = int(raw_width[site,iccd]/xbin)
            }

            # Read NAXIS2 image_height
            keypar (inimage_files[i]//"["//l_sciext//"1]", "i_naxis2", silent+)
            if (keypar.found) {
                image_height = int(keypar.value)
            } else {
                image_height = 0
            }

            file_of_chipgap = mktemp ("file_of_chipgap")
            # Create the image that will be the chip gap when tiled
            mkpattern (file_of_chipgap, pattern="constant", option="replace",\
                v1=l_padvalue, v2=0, pixtype="real", ncols=gapwidth, \
                nlines=image_height)
        }

        # Check for 0 amplifiers
        if (total_num_amplifiers < 1) {
            printlog ("GTILE - ERROR: The file "//inimage_files[i]//\
                " could not be tiled.", l_logfile, verbose+)
            printlog("                 No image extensions were found", \
                l_logfile, verbose+)
            printlog("                 Exiting.", l_logfile, verbose+)

            status = 1
            goto clean
        }

        # Initialise refextn
        refextn = 0

        # Loop through the science extensions and read certain keywords
        for (j=1; j<=total_num_amplifiers; j+=1) {
            # Read the DETSEC, CCDSEC, CRPIX1, DATASEC
            # This could probably be done more compactly

            # DETSEC
            keypar (inimage_files[i]//"["//l_sciext//j//"]", l_key_detsec, \
                silent+)
            if (!keypar.found) {
                printlog ("GTILE - ERROR "//l_key_detsec//" not found. "//\
                    "Exiting.", l_logfile, verbose+)
                status = 1
                goto clean
            }
            detsec[j] = keypar.value
            l_detsec = detsec[j]
            junk = fscanf (l_detsec, "[%d:%d,%d:%d]", tx1, tx2, ty1, ty2)
            detx1[j] = tx1
            detx2[j] = tx2
            dety1[j] = ty1
            dety2[j] = ty2

            # CCDSEC
            keypar (inimage_files[i]//"["//l_sciext//j//"]", l_key_ccdsec, \
                silent+)
            if (!keypar.found) {
                printlog ("GTILE - ERROR: "//l_key_ccdsec//" not found. "//\
                    "Exiting", l_logfile, verbose+)
                status = 1
                goto clean
            }
            ccdsec[j] = keypar.value
            l_ccdsec = ccdsec[j]
            junk = fscanf (l_ccdsec, "[%d:%d,%d:%d]", tx1, tx2, ty1, ty2)
            ccdx1[j] = tx1
            ccdx2[j] = tx2
            ccdy1[j] = ty1
            ccdy2[j] = ty2

            # CCDSEC
            keypar (inimage_files[i]//"["//l_sciext//j//"]", l_key_datasec, \
                silent+)
            if (!keypar.found) {
                printlog ("GTILE - ERROR: "//l_key_datasec//" not found. "//\
                    "Exiting.", l_logfile, verbose+)
                status = 1
                goto clean
            }

            datasec[j] = keypar.value
            l_datasec = datasec[j]
            junk = fscanf (l_datasec, "[%d:%d,%d:%d]", tx1, tx2, ty1, ty2)
            datax1[j] = tx1
            datax2[j] = tx2
            datay1[j] = ty1
            datay2[j] = ty2
            data_width[j] = tx2 - (tx1 -1)

            if (debug) {
                printlog ("Width of data section for "//\
                    inimage_files[i]//"["//l_sciext//j//"] is: "//\
                    data_width[j], logfile, verbose+)
            }

            # CRPIX1
            keypar (inimage_files[i]//"["//l_sciext//j//"]", "CRPIX1", \
                silent+)
            if (keypar.found) {
                junk = fscanf (keypar.value, "%f", dummy_crpix1)
                crpix1_in[j] = dummy_crpix1
            } else {
                # Initialise crpix1
                crpix1_in[j] = 0
            }

            # CRPIX2
            keypar (inimage_files[i]//"["//l_sciext//j//"]", "CRPIX2", \
                silent+)
            if (keypar.found) {
                junk = fscanf (keypar.value, "%f", dummy_crpix2)
                crpix2_in[j] = dummy_crpix2
            } else {
                # Initialise crpix2
                crpix2_in[j] = 0
            }

            if (crpix1_in[j] != 0 && crpix2_in[j] != 0) {
                # Determine the reference extension
                if (crpix1_in[j] > datax1[j] && crpix1_in[j] < datax2[j]) {
                    refextn = j
                }
            }

            if (debug) {
                printlog (l_detsec//" "//l_ccdsec//" "//l_datasec//" "//\
                           crpix1_in[j]//" "//crpix2_in[j], logfile, verbose+)
            }

            # initialize ampsorder to current non sorted amps order
            ampsorder[j] = j

            # Read the amplifier's name
            keypar (inimage_files[i]//"["//l_sciext//j//"]", "AMPNAME", \
                silent+)
            if (keypar.found) {
                amplifiername[j] = keypar.value
            } else {
                amplifiername[j] = "EXTN NUM "//j
            }

        } # End of loop over amplifiers obtaining information

        # Reset REFEXTN if it is equal to 0
        if (refextn == 0) {
            refextn = nint(total_num_amplifiers/2)
        }

        # Get the amplifiers into the correct order
        for (j=2; j<=total_num_amplifiers; j=j+1) {
            for (jj=1; jj<j; jj=jj+1) {
                if (detx1[j] < detx1[jj]) {
                    tmpint = ampsorder[jj]
                    ampsorder[jj] = ampsorder[j]
                    ampsorder[j] = tmpint
                }
            }
        } # End of loop to determine amps order

        # Find number of CCDs used by looping of the amps and
        # comparing detsec and ccdsec also find number_of_amps_per_ccd
        # Inititate them - there will always be 1 ccd and 1 amp at least
        # by this point
        num_ccds = 1
        number_of_ampsperccd[num_ccds] = 1
        for (j=2; j<=total_num_amplifiers; j+=1) {
           if (detx1[ampsorder[j]] > detx1[ampsorder[j-1]] && \
               ccdx1[ampsorder[j]] <= ccdx1[ampsorder[j-1]]) {
               num_ccds += 1
               number_of_ampsperccd[num_ccds] = 1
           } else {
               number_of_ampsperccd[num_ccds] = \
                   number_of_ampsperccd[num_ccds] + 1
           }
        }

        if (debug) {
            printlog ("Number of ccds is: "//num_ccds, logfile, verbose+)
        }

#########
#        ### Left this here in case the usage is to be changed
#        # If there is only amp per CCD, not tiling detector and not stats_only
#        # finish with this image
#        if (!l_fl_tile_det && !l_fl_stats_only && \
#            total_num_amplifiers == num_ccds && !l_fl_stats_only) {
#            printlog ("GTILE WARNING - Only one amplifier per CCD exiting")
#            status = 1
#            goto clean
#        }
#########

        # Find the position of the reference extension in ampsorder
        for (j=1; j<=total_num_amplifiers; j+=1) {
            if (ampsorder[j] == refextn) {
                ordered_refextn = j
            }
        }

        if (debug) {
            printlog ("The reference extension is: "//refextn, logfile, \
                verbose+)
            printlog ("The ordered reference extension is: "//ordered_refextn,\
                logfile, verbose+)
        }

        # If Hamamatsu data the bottom 48 rows may need to be removed if not
        # trimmed
        if (ishamamatsu) {
            # Obtain DETRO1Y (detector region of interest y start value)
            # header keyword value for current image from phu for later
            # comparison. Do it here to stop multiple read of the same
            # keyword in the same file. - MS
            keypar (inimage_files[i]//"[0]","DETRO1Y", silent+)
            if (keypar.found) {
                detroiy_start = int(keypar.value)
            } else {
                # It should exist for Hamamatsu data - if not crash.
                printlog ("GTILE - ERROR: DETRO1Y not found. Exiting.", \
                    logfile, verbose+)
                status = 1
                goto clean
            }
        }

        # Update crpix values if required and if Hamamatsu remove lower 48 rows
        for (j=1; j<=total_num_amplifiers; j+=1) {

            # See if it has been trimmed
            keypar (inimage_files[i]//"["//l_sciext//j//"]", \
                "TRIMMED", silent+)

            # Only need to worry if it hasn't been trimmed and
            # the BIASSEC is on the left of the amplifier
            if (!keypar.found) {
                # Read the BIASSEC
                keypar (inimage_files[i]//"["//l_sciext//j//"]",
                    l_key_biassec, silent+)
                biassec[ampsorder[j]] = keypar.value
                l_biassec = biassec[ampsorder[j]]
                junk = fscanf (l_biassec, "[%d:%d,%d:%d]", tx1, tx2,\
                    ty1, ty2)
                biasx1[ampsorder[j]] = tx1
                biasx2[ampsorder[j]] = tx2
                biasy1[ampsorder[j]] = ty1
                biasy2[ampsorder[j]] = ty2
                bias_width[ampsorder[j]] = tx2 - (tx1 -1)


                # BIASSEC on the left
                # CRPIX is relative to the image pixels in the file
                if (biasx1[j] == 1) {
                    # Subtract bias_width from the CRPIX1 value for this amp
                    crpix1_in[ampsorder[j]] = crpix1_in[ampsorder[j]] \
                        - bias_width[ampsorder[j]]
                }

                # Update CRPIX2 after removing bottom 48 rows
                if (ishamamatsu) {
                    keypar (inimage_files[i]//"["//l_sciext//j//"]",
                        l_key_ccdsum, silent+)
                    if (keypar.found) {
                        print (keypar.value) | scanf ("%d %d", xbin, ybin)
                        if (debug) {
                            print ("XBIN: "//xbin//"YBIN: "//ybin, l_logfile, \
                                verbose+)
                        }
                    } else {
                        xbin = 1
                        ybin = 1
                    }

                    # Calculate binned value for bottom 48 rows
                    row_removal_value_binned = nint(row_removal_value / ybin)

                    # Update CRPIX2 and DATASEC, CCDSEC, DETSEC too
                    # [for hamamatsu data only]
                    if (detroiy_start < row_removal_value_binned) {
                        # CRPIX2
                        crpix2_in[ampsorder[j]] = crpix2_in[ampsorder[j]] \
                            - row_removal_value_binned

                        # DATASEC
                        datay1[ampsorder[j]] = datay1[ampsorder[j]] \
                            + row_removal_value_binned
                        datasec[ampsorder[j]] = "["//datax1[ampsorder[j]]//\
                            ":"//datax2[ampsorder[j]]//","//\
                            datay1[ampsorder[j]]//":"//datay2[ampsorder[j]]//\
                            "]"

                        # CCDSEC
                        ccdy1[ampsorder[j]] = ccdy1[ampsorder[j]] \
                            + row_removal_value_binned
                        ccdsec[ampsorder[j]] = "["//ccdx1[ampsorder[j]]//\
                            ":"//ccdx2[ampsorder[j]]//","//\
                            ccdy1[ampsorder[j]]//":"//ccdy2[ampsorder[j]]//"]"

                        # DETSEC
                        dety1[ampsorder[j]] = dety1[ampsorder[j]] \
                            + row_removal_value_binned
                        detsec[ampsorder[j]] = "["//detx1[ampsorder[j]]//\
                            ":"//detx2[ampsorder[j]]//","//\
                            dety1[ampsorder[j]]//":"//dety2[ampsorder[j]]//"]"

                    } # End of test for Hamamatsu data that is has rows < 48

                } # End of is Hamamatsu loop

            } else {
                istrimmed = yes
            } # End of if not trimmed loop

        } # End of loop over amplifiers obtaining information

        if (debug) {
            for (jj = 1 ; jj <= total_num_amplifiers; jj += 1) {
                printlog ("AMP: "//jj//"---------------", logfile, verbose+)
                printlog ("CRPIX1 is: "//\
                    crpix1_in[ampsorder[jj]], logfile, verbose+)
                printlog ("CRPIX2 is: "//\
                    crpix2_in[ampsorder[jj]], logfile, verbose+)
                printlog ("datasec is: "//\
                    datasec[ampsorder[jj]], logfile, verbose+)
                printlog ("ccdsec is: "//\
                    ccdsec[ampsorder[jj]], logfile, verbose+)
                printlog ("detsec is: "//\
                    detsec[ampsorder[jj]], logfile, verbose+)
            }
        }

        # Copy the phu to the outfile
        if (!l_fl_stats_only) {
            imcopy (inimage_files[i]//"[0]", outimage_files[i], verbose=no)
            if (mdf_exists) {
                tmpmdf = mktemp("tmpmdf")
                tcopy (inimage_files[i]//"["//l_mdf_ext//"]", \
                    outimage_files[i]//"[append,MDF]", \
                    verbose=l_fl_verbose)
            }
        }

        # Check input file against request return ccd
        if (num_ccds == 1 && l_out_ccds != "all") {
            if ((detx1[ampsorder[1]] < det_start[int(l_out_ccds)] || \
                detx1[ampsorder[1]] > det_stop[int(l_out_ccds)]) &&
                (detx2[ampsorder[number_of_ampsperccd[1]]] > \
                det_stop[int(l_out_ccds)] || \
                detx2[ampsorder[number_of_ampsperccd[1]]] < \
                det_start[int(l_out_ccds)])) {

                printlog ("", l_logfile, verbose+)
                printlog ("GTILE - WARNING: CCD "//l_out_ccds//" requested"//\
                    " but not no data with"//\
                    " any information from that CCD in it has been provided",\
                    logfile, verbose+)
                printlog ("                 Continuing anyway...", logfile, \
                    verbose+)
            }
        }

        # Tile the amplifiers into their respective CCDs or total detector
        scanfile = extn_list
        in_extn = ""
        dotile = yes

        # Initialise counters
        num_extns = 0
        num_sci_extns = 0

        # If an MDF is present and not producing a statistics image
        # Update num_extns by 1
        if (mdf_exists && !l_fl_stats_only) {
            num_extns += 1
        }

        # Loop over the extensions to tile
        while (fscan(scanfile,in_extn) != EOF) {

            if (debug) {
                printlog ("Working on extensions called: "//in_extn, logfile, \
                    verbose+)
            }

            # Initiate num_of_this_extension
            num_of_this_extension = 0

            # Initiate jstart for use below in detemining which amps to use
            jstart = 1

            # Initiate out_ampsname
            out_ampsname = ""

            # loop over ccds
            for (j=1;j<=num_ccds;j+=1) {
                if (debug) {
                    printlog ("Number of amplifiers for index "//j//" "//\
                    "of j (num_ccds): "//num_ccds, logfile, verbose+)
                }

                # Set output extension
                extn = ""

                # Determine the amplifiers to use
                # Don't use the line below it assumes the same number of amps
                # per CCD
                # jjstart = j + ((j-1)*(number_of_ampsperccd[j]-1))
                # Use this instead (jstart gets updated below)
                jjstart = jstart
                jjstop = jjstart + number_of_ampsperccd[j]

                if (l_fl_tile_det) {

                    # Initiate the input list for the task that will combine
                    # the amplifiers
                    if (j == 1) {
                        imtile_list = mktemp("tmpimtile_list")
                        crpix1_out = crpix1_in[ampsorder[jjstart]]
                        crpix2_out = crpix2_in[ampsorder[jjstart]]
                    }

                } else {
                    # Reset output extension if not stats_only
                    if (!l_fl_stats_only) {
                        extn = "["//in_extn//j//", append]"
                    }

                    # Initiate the imtile input list
                    imtile_list = mktemp("tmpimtile_list")

                    # Set output crpix values for this ccd
                    crpix1_out = crpix1_in[ampsorder[jjstart]]
                    crpix2_out = crpix2_in[ampsorder[jjstart]]
                }

                # Determine output pixtype
                hselect (inimage_files[i]//"["//in_extn//\
                    ampsorder[jjstart]//"]", \
                    "i_pixtype", "yes") | scan (in_pixtype)
                out_pixtype = substr(l_pixtype[in_pixtype],1,1)

                # Loop over amps for this ccd and create input for the
                # combining task
                for (jj=jjstart; jj<jjstop; jj+=1) {

                    # Obtain th3 current amplifier
                    k = ampsorder[jj]
                    if (debug) {
                        printlog ("Now looking at amplifiers in order: "//\
                        "amp "//k, logfile, verbose+)
                    }

                    # Setup the input list for imtile

                    # Only tile this CCDs worth of amplifiers if requested
                    if (num_ccds > 1 && l_fl_stats_only && l_out_ccds != \
                        "all") {

                        # reset dotile flag
                        dotile = no

                        # Need to check boundaries and set a flag
                        if ((detx1[k] >= det_start[int(l_out_ccds)] && \
                            detx1[k] <= det_stop[int(l_out_ccds)]) &&
                            (detx2[k] <= \
                            det_stop[int(l_out_ccds)] && \
                            detx2[k] >= \
                            det_start[int(l_out_ccds)])) {

                            # Printlog file name to imtile input list
                            print (inimage_files[i]//"["//in_extn//k//"]"//\
                                datasec[k], >> imtile_list)
                            if (jj == jjstop - 1) {
                                # Reset the do tile flag
                                dotile = yes
                            }
                       }

                    } else { # Else do tiling regardless
                        print (inimage_files[i]//"["//in_extn//k//"]"//\
                            datasec[k], >> imtile_list)
                    }

                    if (l_fl_pad && jj == (jjstop - 1) && j < num_ccds) {
                        print (file_of_chipgap//"[0]", >> imtile_list)
                    }

                    # Update the output amp name list only if it is to be used
                    if (dotile) {
                        out_ampsname = out_ampsname//","//amplifiername[k]
                    }

                    if (debug) {
                        printlog ("Current out_ampsname is: "//out_ampsname, \
                            logfile, verbose+)
                    }

                } # End of configuring paste variable

                # Do the tiling!
                if ((!l_fl_tile_det && dotile) \
                    || (l_fl_tile_det && j==num_ccds)) {

                    if (debug) {
                        printlog ("In first section of the dotile loop", \
                            logfile, verbose+)
                    }

                    # Only call imjoin if more than one amp - else imcopy
                    if (!l_fl_tile_det && number_of_ampsperccd[j] == 1) {
                        if (debug) {
                            printlog ("Only imcopying the file", logfile, \
                                verbose+)
                            printlog ("Output name is: "//outimage_files[i]//\
                                extn, logfile, verbose+)

                        }
                        scanfileII = imtile_list
                        while (fscan(scanfileII, in_tile_file) != EOF) {
                            imcopy (in_tile_file, outimage_files[i]//extn, \
                                verbose=l_fl_verbose)#, >& "dev$null")
                            # Update the extension version - only needed
                            # for raw frames when not producing a statistics
                            # image
                            num_of_this_extension += 1
                            if (in_extn == "") {
                                if (!l_fl_stats_only) {
                                    gemhedit (outimage_files[i]//extn, \
                                        "EXTVER", num_of_this_extension, "", \
                                        delete-)
                                    # Have to update EXTNAME too to get arround
                                    # Pyraf not liking the imcopy /append
                                    # of raw data frames
                                    gemhedit (outimage_files[i]//extn, \
                                        "EXTNAME", "TEST", "", \
                                        delete-)
                                    if (debug) {
                                        printlog ("Updated EXTVER to: "//\
                                            num_of_this_extension, \
                                            l_logfile, verbose+)
                                    }
                                }

                            }

                            # Slight code duplication to update DATASEC - MS

                            # Obtain the keyword value for datasec
                            keypar (outimage_files[i]//extn, l_key_datasec, \
                                silent+)
                            if (keypar.found) {
                                l_sec = keypar.value
                                junk = fscanf (l_sec, "[%d:%d,%d:%d]", tx1,\
                                    tx2, ty1, ty2)

                                # Extract the individual values
                                secx1 = tx1
                                secx2 = tx2
                                secy1 = ty1
                                secy2 = ty2

                                # Obtain the NAXIS1 value for this extension to
                                # update DATASEC, DETSEC and CCDSEC
                                keypar (outimage_files[i]//extn, "i_naxis1", \
                                    silent+)
                                if (keypar.found) {
                                    image_width = int(keypar.value)
                                } else {
                                    image_width = 0
                                }

                                # Reset to the new DATASEC
                                secx1 = 1
                                secx2 = image_width

                                # Define the section value
                                l_sec = "["//secx1//\
                                    ":"//secx2//","//\
                                    secy1//":"//secy2//"]"

                                # Update the keyword
                                gemhedit (outimage_files[i]//extn, \
                                    l_key_datasec, \
                                    l_sec, "", delete-)

                            } else {
                                printlog ("GTILE - WARNING: "//l_key_datasec//\
                                    " not found", l_logfile, verbose+)
                                printlog ("               Ignoring and "//\
                                    "going to next keyword", l_logfile, \
                                    verbose+)
                            }
                        }
                        scanfileII = ""
                    } else {

                        if (debug) {
                            printlog ("Writing to "//outimage_files[i]//extn, \
                                logfile, verbose+)
                        }

                        # Doing the joining
                        imjoin (input="@"//imtile_list, \
                            output=outimage_files[i]//extn, join_dim=1,\
                            pixtype=out_pixtype, verbose=l_fl_verbose)

                        # Update the extension version - only needed
                        # for raw frames when not producing a statistics image
                        num_of_this_extension += 1
                        if (in_extn == "") {
                            if (!l_fl_stats_only) {
                                gemhedit (outimage_files[i]//extn, "EXTVER", \
                                    num_of_this_extension, "", delete-)
                                if (debug) {
                                    printlog ("Updated EXTVER to: "//\
                                        num_of_this_extension, \
                                            l_logfile, verbose+)
                                }
                            }

                        }

                        # Update the amplifier name in output extension
                        if (substr(out_ampsname,1,1) == ",") {
                            out_ampsname=substr(out_ampsname,2, \
                                strlen(out_ampsname))
                        }

                        # Update AMPNAME, CRPIX, DATASEC, DETSEC and CCDSEC
                        # keywords here

                        # AMPNAME
                        # Only do this if not DQ plane
                        if (in_extn != "DQ,") {
                            gemhedit (outimage_files[i]//extn, "AMPNAME",
                                out_ampsname, "", delete-)

                            # CRPIX1
                            gemhedit (outimage_files[i]//extn, "CRPIX1",
                                crpix1_out, "", delete-)

                            # CRPIX2
                            gemhedit (outimage_files[i]//extn, "CRPIX2",
                                crpix2_out, "", delete-)
                        }

                        # Obtain the NAXIS(1/2) value for this extension to
                        # update DATASEC, DETSEC and CCDSEC
                        keypar (outimage_files[i]//extn, "i_naxis1", silent+)
                        if (keypar.found) {
                            image_width = int(keypar.value)
                        } else {
                            image_width = 0
                        }

                        keypar (outimage_files[i]//extn, "i_naxis2", silent+)
                        if (keypar.found) {
                            image_height = int(keypar.value)
                        } else {
                            image_height = 0
                        }

                        # Obtain the binning for the image to adjust
                        # CCDSEC and DETSEC properly
                        keypar (outimage_files[i]//extn, l_key_ccdsum, silent+)
                        if (keypar.found) {
                            print (keypar.value) | scanf ("%d %d", xbin, ybin)
                        } else {
                            xbin = 1
                            ybin = 1
                        }

                        # imjoin uses the header information from the 1st file
                        # it receives in that call as the header for the output

                        # Loop over array describing header keywords
                        for (kk=1; kk<=3; kk+=1) {

                            # Set the header keyword to read
                            l_key = array_keys[kk]

                            # Obtain the keyword value
                            keypar (outimage_files[i]//extn, l_key, silent+)
                            if (keypar.found) {
                                l_sec = keypar.value
                                junk = fscanf (l_sec, "[%d:%d,%d:%d]", tx1,\
                                    tx2, ty1, ty2)

                                # Extract the individual values
                                secx1 = tx1
                                secx2 = tx2
                                secy1 = ty1
                                secy2 = ty2

                                # Update the key value width
                                if (l_key != "DATASEC") {
                                    secx2 = (secx1 - 1) + (image_width * xbin)
                                } else {
                                    secx1 = 1
                                    secx2 = image_width
                                }

                                # A slight cludge here - could make it more
                                # elegant I think - MS
                                # Update the y values of the sections
                                if (ishamamatsu && !istrimmed) {
                                    if (detroiy_start<row_removal_value_binned){
                                        if (l_key != "DATASEC") {
                                            # If not DATASEC update y1
                                            secy1 += (row_removal_value_binned-\
                                                (detroiy_start - 1))
                                        } else {
                                            # Update y2 if DATASEC
                                            secy2 = image_height
                                        }
                                    }
                                }

                                # Define the section value
                                l_sec = "["//secx1//\
                                    ":"//secx2//","//\
                                    secy1//":"//secy2//"]"

                                # Update the keyword
                                gemhedit (outimage_files[i]//extn, l_key, \
                                    l_sec, "", delete-)

                            } else {
                                printlog ("GTILE - WARNING: "//l_key//" not"//\
                                    " found", l_logfile, verbose+)
                                printlog ("               Ignoring and "//\
                                    "going to next keyword", l_logfile, verbose+)
                            }
                        }
                    }

                    # Reset out_ampsname
                    out_ampsname = ""

                    # Reset the LTV1 and 2 keywords using wcsreset
                    # wcs=physicsal
                    # This is so logical and physical coordinates relate to
                    # the new file not any images they may have come from
                    wcsreset (outimage_files[i]//extn, wcs="physical",\
                        verbose-)

                    # Count the total number of science extensions
                    if (in_extn == l_sciext) {
                        num_sci_extns += 1
                    }

                    # Count the total number of extensions in the output
                    num_extns += 1

                } # End of tiling

                # Update jstart to be correct and not expect the same number
                # of amps per ccd
                jstart = jstart + number_of_ampsperccd[j]

            } # End of pasting and looping over ccds

            # Update NEXTEND and NSCIETN here
            gemhedit (outimage_files[i]//"[0]", "NEXTEND", num_extns, "",\
                delete-)
            gemhedit (outimage_files[i]//"[0]", "NSCIEXT", num_sci_extns, "",\
                delete-)

            # Due to PyRAF not liking to append imjoined raw frames to
            # the output, update EXTVER of the output frames to -1 here - MS
            # If input was raw should be no other extensions other than
            # those joined earlier; though will double check for MDFs - MS
            # Only needs to be done for tiled raw frame with non-statistics
            # output
            if (in_extn == "" && !l_fl_stats_only) {
                for (j=1;j<=num_ccds;j+=1) {
                    current_extn = j
                    if (mdf_exists) {
                        current_extn += 1
                    }
                    if (debug) {
                        printlog ("current_extn for "//j//"is: "//\
                            current_extn, l_logfile, verbose+)
                    }
                    gemhedit (outimage_files[i]//"["//current_extn//"]", \
                        "EXTVER", -1, "", delete-)

                    if (gemhedit.status != 0) {
                        printlog ("GTILE - ERROR: GEMHEDIT fatal error",\
                           l_logfile, verbose+)
                    }

                    # This test and subsequent update is here for
                    # the fact that Pyraf doesn't like ot append raw data
                    # especially when imcopying
                    keypar (outimage_files[i]//"["//current_extn//"]", \
                        "EXTNAME", silent+)
                    if (keypar.value == "TEST") {
                        gemhedit (outimage_files[i]//"["//current_extn//"]", \
                            "EXTNAME", "", "", delete-)
                    }

                    if (gemhedit.status != 0) {
                        printlog ("GTILE - ERROR: GEMHEDIT fatal error",\
                           l_logfile, verbose+)
                    }
                }
            }

            # Can only delete these tmpimtile_list files here as they are
            # needed in case of l_fl_stats_only. Saves a number of if loops -
            # MS
            delete ("tmpimtile_list*", verify-, >& "dev$null")

        } # End of loop over extension types

        delete (extn_list, verify-, >& "dev$null")

        if (l_fl_pad) {
            imdelete (file_of_chipgap, verify=no, >& "dev$null")
        }

        # Write GTILE time stmap to header
        gemdate ()
        gemhedit (outimage_files[i]//"[0]", "GTILE", gemdate.outdate, \
            "UT Time stamp for GTILE", delete-)
        gemhedit (outimage_files[i]//"[0]", "GEM-TLM", gemdate.outdate, \
            "UT Last modification with GEMINI", delete-)

    } # End of loop over input

    ####################
    # End of hard work #
    ####################

clean:

    # Clean up a bit - delete inlist and outlist
    scanfile = ""
    delete (inlist//", "//outlist, verify-, >& "dev$null")

exit:

    printlog ("", l_logfile, l_fl_verbose)
    gemdate ()
    # Printlog finish time
    printlog ("GTILE - Finished: "//gemdate.outdate, logfile, l_fl_verbose)
    printlog ("", l_logfile, l_fl_verbose)

    if (status == 0) {
        printlog ("GTILE - Exit status: GOOD", logfile, l_fl_verbose)
    } else {
        printlog ("GTILE - Exit status: ERROR", logfile, l_fl_verbose)
    }

    printlog ("", l_logfile, verbose+)

end

